#include <stdio.h>
#include <stdlib.h>

// Copies string ct to string s and returns s.
char *strcopy(char *s, const char *ct)
{
	char *r = s; // Need to store initial value of s for return statments since s is modified.
	
	while (*s++ = *ct++); // i.e. while (*ct != '\0') { *s = *ct; s++; ct++; } ++s = '\0';
	
	return r;
}

// Concatenates string ct to the end of string s. Returns s.
char *strconcat(char *s, const char *ct)
{
	char *r = s; // Need to store initial value of s for return statments since s is modified.
	
	// Find '\0' of s.
	
	while (*s) s++; // i.e. while (*s != '\0') s++; The pointer is incremented after the test of '\0' so as not to skip '\0' character.
	
	// Copy ct into s, starting from '\0' of s.
	
	while (*s++ = *ct++); // i.e. strcpy(s, ct);
	
	return r;
}

// Returns the length of string cs, including the '\0' character.
int strlength(const char *cs)
{
	int i = 1; // Initialize with 1 to count '\0' character.
	
	while (*cs++) i++; // i.e. while (*cs != '\0') i++;
	
	return i;
}

// Compares string cs to string ct. Returns -1 if cs < ct, 0 if cs == ct and 1 if cs > ct.
// The comparison is case INSENSITIVE.

// Macro for converting an uppercase character to lowercase.
#define lcase(x) ((x) >= 'A' || (x) <= 'Z' ? (x) + ('a' - 'A') : (x))

int strcompare(const char *cs, const char *ct)
{
	while (*cs && *ct)
	{
		if (lcase(*cs) > lcase(*ct)) return 1;
		if (lcase(*cs) < lcase(*ct)) return -1;
		
		cs++;
		ct++;
	}
	
	// *cs == '\0' || *ct == '\0'
	
	if (lcase(*cs) == lcase(*ct)) return 0;
	if (*cs) return 1;
	return -1;
}

// Returns a pointer to the first occurrence of string ct in string cs, or NULL if not found.
char *stringstring(const char *cs, const char *ct)
{
	const char *pcs; // Advances through characters 
	const char *pct;
		
	while (*cs) // i.e. while (*cs != '\0') ...
	{
		
		pcs = cs;
		pct = ct;
		
		while (*pcs && *pct && *pcs == *pct) // i.e. while (*pcs != '\0' && *pct != '\0' && *pcs == *pct) ...
		{ 
			pcs++; 
			pct++; 
		}
		
		// *pcs == '\0' || *pct == '\0' || *pcs != *pct
		
		if (!*pct) return (char *) cs; // i.e. if (*pct == '\0') we have reached the end of ct. Found!
		if (!*pcs) return NULL; // i.e. if (*pcs == '\0') we have reached the end of cs. Not found.
		
		cs++;
	}
	
	// *cs == '\0'
	
	return NULL;
}

// Returns the numeric value of string cs.
double val(const char *cs)
{
	long double r = 0;
	short sign = 1;
	double frac = 1;
	short has_frac = 0;
	
	// Take into account leading negative sign if present.
	if (*cs == '-')
	{
		sign = -1;
		cs++;
	}
	
	// Skip leading positive sign if present ('sign' already initialized with 1).
	if (*cs == '+')
	{
		cs++;
	}
	
	while (*cs) // i.e. while (*cs != '\0') ...
	{
		// Make note of decimal point, if encountered, and then skip it.
		if (*cs == '.')
		{
			has_frac = 1;
		}
		else
		{
			// r * 10 'shifts' result to the left, making room for next digit.
			// *cs - '0' converts the ASCII code of a digit stored in *cs to its numeric value.
			r = r * 10 + *cs - '0';
			
			// If decimal point has been encountered we need to increase denominator by a factor of 10.
			if (has_frac) 
				frac *= 10; // i.e. frac = frac * 10
		}
		
		cs++;
	}
	
	// For positive integer, sign == 1 and frac == 1 so the result is simply 'r'.
	// For negative integer, sign == -1 and frac == 1 so the result is simply '-r'.
	// For positive real number, sign == 1 and frac >= 10 so the result is 'r/frac'.
	// For negative real number, sign == -1 and frac >= 10 so the result is '-r/frac'.
	return (double) (sign * r / frac);
}





int main(void)
{

	char *s1 = "String";
	char *s2 = "ring";
	
	char *s3 = (char *) malloc(11 * sizeof(char));
	char *s4;
	int l, c;

	s3 = strcopy(s3, s1);
	printf("strcopy(s3, \"%s\") = \"%s\"\n", s1, s3);
	
	c = strcompare(s3, s1);
	printf("strcompare(\"%s\", \"%s\") = %d\n", s3, s1, c);
	
	s3 = strconcat(s3, s2);
	printf("strconcat(s3, \"%s\") = \"%s\"\n", s2, s3);
	
	l = strlength(s3);
	printf("strlength(\"%s\") = %d\n", s3, l);
	
	c = strcompare(s1, s2);
	printf("strcompare(\"%s\", \"%s\") = %d\n", s1, s2, c);
	
	c = strcompare(s2, s1);
	printf("strcompare(\"%s\", \"%s\") = %d\n", s2, s1, c);
	
	s4 = stringstring(s1, s2);
	printf("stringstring(\"%s\", \"%s\") = ", s1, s2);
	
	if (s4) printf("\"");
	
	printf("%s", s4 ? s4 : "NULL");
	
	if (s4) printf("\"");
	
	printf("\n");
	
	s4 = stringstring(s3, s1);
	printf("stringstring(\"%s\", \"%s\") = ", s3, s1);
	
	if (s4) printf("\"");
	
	printf("%s", s4 ? s4 : "NULL");
	
	if (s4) printf("\"");
	
	printf("\n");
	
	s4 = stringstring(s1, s3);
	printf("stringstring(\"%s\", \"%s\") = ", s1, s3);
	
	if (s4) printf("\"");
	
	printf("%s", s4 ? s4 : "NULL");
	
	if (s4) printf("\"");
	
	printf("\n");
	
	printf("val(\"%s\") = %lf\n", "+11111.000005", val("+11111.000005"));
	printf("val(\"%s\") = %lf\n", "-11111.000005", val("-11111.000005"));
	printf("val(\"%s\") = %lf\n", "11111.000005", val("11111.000005"));
	printf("val(\"%s\") = %lf\n", "12345", val("12345"));
	
	system ("PAUSE");
	return 0;
}
